@@ -14,6 +14,7 @@
 #import "GPBUnknownField.h"
 #import "GPBUnknownFieldSet_PackagePrivate.h"
 #import "GPBUnknownFields.h"
+#import "GPBUtilities.h"
 #import "GPBUtilities_PackagePrivate.h"
 #import "GPBWireFormat.h"
 #import "objectivec/Tests/Unittest.pbobjc.h"
@@ -559,7 +560,7 @@ static NSData* DataForGroupsOfDepth(NSUInteger depth) {
   [self assertReadByteToEndGroupFails:bytes(35, 25, 0xf0, 0xde, 0xbc, 0x9a, 0x78, 0x56,
                                             0x34)];  // truncated fixed64
 
-  // Mising end group
+  // Missing end group
   [self assertReadByteToEndGroupFails:bytes(35)];
   [self assertReadByteToEndGroupFails:bytes(35, 8, 1)];
   [self assertReadByteToEndGroupFails:bytes(35, 43)];
