@@ -432,7 +432,10 @@ err:
   return ok;
 }
 
+static PyObject* PyUpb_Message_Clear(PyUpb_Message* self);
+
 static bool PyUpb_Message_InitMessageAttribute(PyObject* _self, PyObject* name,
+                                               const upb_FieldDef* field,
                                                PyObject* value) {
   PyObject* submsg = PyUpb_Message_GetAttr(_self, name);
   if (!submsg) return -1;
@@ -444,12 +447,32 @@ static bool PyUpb_Message_InitMessageAttribute(PyObject* _self, PyObject* name,
     Py_XDECREF(tmp);
   } else if (PyDict_Check(value)) {
     assert(!PyErr_Occurred());
-    ok = PyUpb_Message_InitAttributes(submsg, NULL, value) >= 0;
+    const upb_MessageDef* msgdef = upb_FieldDef_MessageSubDef(field);
+    if (upb_MessageDef_WellKnownType(msgdef) == kUpb_WellKnown_Struct) {
+      ok = PyObject_CallMethod(submsg, "_internal_assign", "O", value);
+      if (!ok && PyDict_Size(value) == 1 &&
+          PyDict_Contains(value, PyUnicode_FromString("fields"))) {
+        // Fall back to init as normal message field.
+        PyErr_Clear();
+        PyObject* tmp = PyUpb_Message_Clear((PyUpb_Message*)submsg);
+        Py_DECREF(tmp);
+        ok = PyUpb_Message_InitAttributes(submsg, NULL, value) >= 0;
+      }
+    } else {
+      ok = PyUpb_Message_InitAttributes(submsg, NULL, value) >= 0;
+    }
   } else {
-    const upb_MessageDef* m = PyUpb_Message_GetMsgdef(_self);
-    PyErr_Format(PyExc_TypeError, "Message must be initialized with a dict: %s",
-                 upb_MessageDef_FullName(m));
-    ok = false;
+    const upb_MessageDef* msgdef = upb_FieldDef_MessageSubDef(field);
+    if (upb_MessageDef_WellKnownType(msgdef) != kUpb_WellKnown_Unspecified &&
+        PyObject_HasAttrString(submsg, "_internal_assign")) {
+      ok = PyObject_CallMethod(submsg, "_internal_assign", "O", value);
+    } else {
+      const upb_MessageDef* m = PyUpb_Message_GetMsgdef(_self);
+      PyErr_Format(PyExc_TypeError,
+                   "Message must be initialized with a dict: %s",
+                   upb_MessageDef_FullName(m));
+      ok = false;
+    }
   }
   Py_DECREF(submsg);
   return ok;
@@ -502,7 +525,7 @@ int PyUpb_Message_InitAttributes(PyObject* _self, PyObject* args,
     } else if (upb_FieldDef_IsRepeated(f)) {
       if (!PyUpb_Message_InitRepeatedAttribute(_self, name, value)) return -1;
     } else if (upb_FieldDef_IsSubMessage(f)) {
-      if (!PyUpb_Message_InitMessageAttribute(_self, name, value)) return -1;
+      if (!PyUpb_Message_InitMessageAttribute(_self, name, f, value)) return -1;
     } else {
       if (!PyUpb_Message_InitScalarAttribute(msg, f, value, arena)) return -1;
     }
@@ -757,6 +780,13 @@ static PyObject* PyUpb_Message_RichCompare(PyObject* _self, PyObject* other,
     Py_INCREF(Py_NotImplemented);
     return Py_NotImplemented;
   }
+  const upb_MessageDef* msgdef = _PyUpb_Message_GetMsgdef(self);
+  upb_WellKnown wkt = upb_MessageDef_WellKnownType(msgdef);
+  if ((wkt == kUpb_WellKnown_ListValue && PyList_Check(other)) ||
+      (wkt == kUpb_WellKnown_Struct && PyDict_Check(other))) {
+    return PyObject_CallMethod(_self, "_internal_compare", "O", other);
+  }
+
   if (!PyObject_TypeCheck(other, Py_TYPE(self))) {
     Py_INCREF(Py_NotImplemented);
     return Py_NotImplemented;
@@ -935,9 +965,9 @@ int PyUpb_Message_SetFieldValue(PyObject* _self, const upb_FieldDef* field,
   PyUpb_Message* self = (void*)_self;
   assert(value);
 
-  if (upb_FieldDef_IsSubMessage(field) || upb_FieldDef_IsRepeated(field)) {
+  if (upb_FieldDef_IsRepeated(field)) {
     PyErr_Format(exc,
-                 "Assignment not allowed to message, map, or repeated "
+                 "Assignment not allowed to map, or repeated "
                  "field \"%s\" in protocol message object.",
                  upb_FieldDef_Name(field));
     return -1;
@@ -945,6 +975,25 @@ int PyUpb_Message_SetFieldValue(PyObject* _self, const upb_FieldDef* field,
 
   PyUpb_Message_EnsureReified(self);
 
+  if (upb_FieldDef_IsSubMessage(field)) {
+    const upb_MessageDef* msgdef = upb_FieldDef_MessageSubDef(field);
+    if (upb_MessageDef_WellKnownType(msgdef) != kUpb_WellKnown_Unspecified) {
+      PyObject* sub_message = PyUpb_Message_GetFieldValue(_self, field);
+      if (PyObject_HasAttrString(sub_message, "_internal_assign")) {
+        PyObject* ok =
+            PyObject_CallMethod(sub_message, "_internal_assign", "O", value);
+        if (!ok) return -1;
+        Py_DECREF(ok);
+        return 0;
+      }
+    }
+    PyErr_Format(exc,
+                 "Assignment not allowed to message "
+                 "field \"%s\" in protocol message object.",
+                 upb_FieldDef_Name(field));
+    return -1;
+  }
+
   upb_MessageValue val;
   upb_Arena* arena = PyUpb_Arena_Get(self->arena);
   if (!PyUpb_PyToUpb(value, field, &val, arena)) {
@@ -1039,9 +1088,38 @@ static PyObject* PyUpb_Message_HasField(PyObject* _self, PyObject* arg) {
 
   if (PyUpb_Message_IsStub(self)) Py_RETURN_FALSE;
 
-  return PyBool_FromLong(field ? upb_Message_HasFieldByDef(self->ptr.msg, field)
-                               : upb_Message_WhichOneof(self->ptr.msg, oneof) !=
-                                     NULL);
+  return PyBool_FromLong(
+      field ? upb_Message_HasFieldByDef(self->ptr.msg, field)
+            : upb_Message_WhichOneofByDef(self->ptr.msg, oneof) != NULL);
+}
+
+static PyObject* PyUpb_Message_Contains(PyObject* _self, PyObject* arg) {
+  const upb_MessageDef* msgdef = PyUpb_Message_GetMsgdef(_self);
+  switch (upb_MessageDef_WellKnownType(msgdef)) {
+    case kUpb_WellKnown_Struct: {
+      // For WKT Struct, check if the key is in the fields.
+      PyUpb_Message* self = (void*)_self;
+      if (PyUpb_Message_IsStub(self)) Py_RETURN_FALSE;
+      upb_Message* msg = PyUpb_Message_GetMsg(self);
+      const upb_FieldDef* f = upb_MessageDef_FindFieldByName(msgdef, "fields");
+      const upb_Map* map = upb_Message_GetFieldByDef(msg, f).map_val;
+      const upb_MessageDef* entry_m = upb_FieldDef_MessageSubDef(f);
+      const upb_FieldDef* key_f = upb_MessageDef_Field(entry_m, 0);
+      upb_MessageValue u_key;
+      if (!PyUpb_PyToUpb(arg, key_f, &u_key, NULL)) return NULL;
+      return PyBool_FromLong(upb_Map_Get(map, u_key, NULL));
+    }
+    case kUpb_WellKnown_ListValue: {
+      // For WKT ListValue, check if the key is in the items.
+      PyUpb_Message* self = (void*)_self;
+      if (PyUpb_Message_IsStub(self)) Py_RETURN_FALSE;
+      PyObject* items = PyObject_CallMethod(_self, "items", NULL);
+      return PyBool_FromLong(PySequence_Contains(items, arg));
+    }
+    default:
+      // For other messages, check with HasField.
+      return PyUpb_Message_HasField(_self, arg);
+  }
 }
 
 static PyObject* PyUpb_Message_FindInitializationErrors(PyObject* _self,
@@ -1200,8 +1278,6 @@ PyObject* PyUpb_Message_MergeFrom(PyObject* self, PyObject* arg) {
   Py_RETURN_NONE;
 }
 
-static PyObject* PyUpb_Message_Clear(PyUpb_Message* self);
-
 static PyObject* PyUpb_Message_CopyFrom(PyObject* _self, PyObject* arg) {
   if (_self->ob_type != arg->ob_type) {
     PyErr_Format(PyExc_TypeError,
@@ -1257,6 +1333,9 @@ PyObject* PyUpb_Message_MergeFromString(PyObject* _self, PyObject* arg) {
     int err = PyBytes_AsStringAndSize(bytes, &buf, &size);
     (void)err;
     assert(err >= 0);
+  } else if (PyByteArray_Check(arg)) {
+    buf = PyByteArray_AsString(arg);
+    size = PyByteArray_Size(arg);
   } else if (PyBytes_AsStringAndSize(arg, &buf, &size) < 0) {
     return NULL;
   }
@@ -1275,7 +1354,9 @@ PyObject* PyUpb_Message_MergeFromString(PyObject* _self, PyObject* arg) {
       upb_Decode(buf, size, self->ptr.msg, layout, extreg, options, arena);
   Py_XDECREF(bytes);
   if (status != kUpb_DecodeStatus_Ok) {
-    PyErr_Format(state->decode_error_class, "Error parsing message");
+    PyErr_Format(state->decode_error_class,
+                 "Error parsing message with type '%s'",
+                 upb_MessageDef_FullName(msgdef));
     return NULL;
   }
   PyUpb_Message_SyncSubobjs(self);
@@ -1392,7 +1473,7 @@ static PyObject* PyUpb_Message_ClearField(PyObject* _self, PyObject* arg) {
     return NULL;
   }
 
-  if (o) f = upb_Message_WhichOneof(self->ptr.msg, o);
+  if (o) f = upb_Message_WhichOneofByDef(self->ptr.msg, o);
   if (f) PyUpb_Message_DoClearField(_self, f);
   Py_RETURN_NONE;
 }
@@ -1587,7 +1668,7 @@ static PyObject* PyUpb_Message_WhichOneof(PyObject* _self, PyObject* name) {
   }
   upb_Message* msg = PyUpb_Message_GetIfReified(_self);
   if (!msg) Py_RETURN_NONE;
-  const upb_FieldDef* f = upb_Message_WhichOneof(msg, o);
+  const upb_FieldDef* f = upb_Message_WhichOneofByDef(msg, o);
   if (!f) Py_RETURN_NONE;
   return PyUnicode_FromString(upb_FieldDef_Name(f));
 }
@@ -1641,6 +1722,8 @@ static PyMethodDef PyUpb_Message_Methods[] = {
     // TODO
     //{ "__unicode__", (PyCFunction)ToUnicode, METH_NOARGS,
     //  "Outputs a unicode representation of the message." },
+    {"__contains__", PyUpb_Message_Contains, METH_O,
+     "Checks if a message field is set."},
     {"ByteSize", (PyCFunction)PyUpb_Message_ByteSize, METH_NOARGS,
      "Returns the size of the message in bytes."},
     {"Clear", (PyCFunction)PyUpb_Message_Clear, METH_NOARGS,
@@ -1942,7 +2025,7 @@ static int PyUpb_MessageMeta_Traverse(PyObject* self, visitproc visit,
   return cpython_bits.type_traverse(self, visit, arg);
 }
 
-static int PyUpb_MessageMeta_Clear(PyObject* self, visitproc visit, void* arg) {
+static int PyUpb_MessageMeta_Clear(PyObject* self) {
   PyUpb_MessageMeta* meta = PyUpb_GetMessageMeta(self);
   Py_CLEAR(meta->py_message_descriptor);
   return cpython_bits.type_clear(self);
